#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/

#include "/builddir/build/BUILD/python-xeddsa-0.6.0-beta/ref10/crypto_sign/module.h"

/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 121), // int()(ge_p3 *, unsigned char const *)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 116), // ge_p3 *
/*  2 */ _CFFI_OP(_CFFI_OP_POINTER, 122), // unsigned char const *
/*  3 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  4 */ _CFFI_OP(_CFFI_OP_FUNCTION, 121), // int()(int32_t const *)
/*  5 */ _CFFI_OP(_CFFI_OP_POINTER, 118), // int32_t const *
/*  6 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  7 */ _CFFI_OP(_CFFI_OP_FUNCTION, 121), // int()(unsigned char *, unsigned char *)
/*  8 */ _CFFI_OP(_CFFI_OP_POINTER, 122), // unsigned char *
/*  9 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 10 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 11 */ _CFFI_OP(_CFFI_OP_FUNCTION, 121), // int()(unsigned char *, unsigned long long *, unsigned char const *, unsigned long long, unsigned char const *)
/* 12 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 13 */ _CFFI_OP(_CFFI_OP_POINTER, 15), // unsigned long long *
/* 14 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 15 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 12), // unsigned long long
/* 16 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 17 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 18 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(ge_cached *, ge_p3 const *)
/* 19 */ _CFFI_OP(_CFFI_OP_POINTER, 113), // ge_cached *
/* 20 */ _CFFI_OP(_CFFI_OP_POINTER, 116), // ge_p3 const *
/* 21 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 22 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(ge_p1p1 *, ge_p2 const *)
/* 23 */ _CFFI_OP(_CFFI_OP_POINTER, 114), // ge_p1p1 *
/* 24 */ _CFFI_OP(_CFFI_OP_POINTER, 115), // ge_p2 const *
/* 25 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 26 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(ge_p1p1 *, ge_p3 const *)
/* 27 */ _CFFI_OP(_CFFI_OP_NOOP, 23),
/* 28 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 29 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 30 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(ge_p1p1 *, ge_p3 const *, ge_cached const *)
/* 31 */ _CFFI_OP(_CFFI_OP_NOOP, 23),
/* 32 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 33 */ _CFFI_OP(_CFFI_OP_POINTER, 113), // ge_cached const *
/* 34 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 35 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(ge_p1p1 *, ge_p3 const *, ge_precomp const *)
/* 36 */ _CFFI_OP(_CFFI_OP_NOOP, 23),
/* 37 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 38 */ _CFFI_OP(_CFFI_OP_POINTER, 117), // ge_precomp const *
/* 39 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 40 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(ge_p2 *)
/* 41 */ _CFFI_OP(_CFFI_OP_POINTER, 115), // ge_p2 *
/* 42 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 43 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(ge_p2 *, ge_p1p1 const *)
/* 44 */ _CFFI_OP(_CFFI_OP_NOOP, 41),
/* 45 */ _CFFI_OP(_CFFI_OP_POINTER, 114), // ge_p1p1 const *
/* 46 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 47 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(ge_p2 *, ge_p3 const *)
/* 48 */ _CFFI_OP(_CFFI_OP_NOOP, 41),
/* 49 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 50 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 51 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(ge_p2 *, unsigned char const *, ge_p3 const *, unsigned char const *)
/* 52 */ _CFFI_OP(_CFFI_OP_NOOP, 41),
/* 53 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 54 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 55 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 56 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 57 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(ge_p3 *)
/* 58 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 59 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 60 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(ge_p3 *, ge_p1p1 const *)
/* 61 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 62 */ _CFFI_OP(_CFFI_OP_NOOP, 45),
/* 63 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 64 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(ge_p3 *, unsigned char const *)
/* 65 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 66 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 67 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 68 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(ge_precomp *)
/* 69 */ _CFFI_OP(_CFFI_OP_POINTER, 117), // ge_precomp *
/* 70 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 71 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(int32_t *)
/* 72 */ _CFFI_OP(_CFFI_OP_POINTER, 118), // int32_t *
/* 73 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 74 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(int32_t *, int32_t const *)
/* 75 */ _CFFI_OP(_CFFI_OP_NOOP, 72),
/* 76 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 77 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 78 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(int32_t *, int32_t const *, int32_t const *)
/* 79 */ _CFFI_OP(_CFFI_OP_NOOP, 72),
/* 80 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 81 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 82 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 83 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(int32_t *, int32_t const *, unsigned int)
/* 84 */ _CFFI_OP(_CFFI_OP_NOOP, 72),
/* 85 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 86 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8), // unsigned int
/* 87 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 88 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(int32_t *, unsigned char const *)
/* 89 */ _CFFI_OP(_CFFI_OP_NOOP, 72),
/* 90 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 91 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 92 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(unsigned char *)
/* 93 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 94 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 95 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(unsigned char *, ge_p2 const *)
/* 96 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 97 */ _CFFI_OP(_CFFI_OP_NOOP, 24),
/* 98 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 99 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(unsigned char *, ge_p3 const *)
/* 100 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 101 */ _CFFI_OP(_CFFI_OP_NOOP, 20),
/* 102 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 103 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(unsigned char *, int32_t const *)
/* 104 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 105 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 106 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 107 */ _CFFI_OP(_CFFI_OP_FUNCTION, 123), // void()(unsigned char *, unsigned char const *, unsigned char const *, unsigned char const *)
/* 108 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 109 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 110 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 111 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 112 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 113 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 0), // ge_cached
/* 114 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 1), // ge_p1p1
/* 115 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 2), // ge_p2
/* 116 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 3), // ge_p3
/* 117 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 4), // ge_precomp
/* 118 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 21), // int32_t
/* 119 */ _CFFI_OP(_CFFI_OP_ARRAY, 118), // int32_t[10]
/* 120 */ (_cffi_opcode_t)(10),
/* 121 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7), // int
/* 122 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 4), // unsigned char
/* 123 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_ge_cached(ge_cached *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { int32_t(*tmp)[10] = &p->YplusX; (void)tmp; }
  { int32_t(*tmp)[10] = &p->YminusX; (void)tmp; }
  { int32_t(*tmp)[10] = &p->Z; (void)tmp; }
  { int32_t(*tmp)[10] = &p->T2d; (void)tmp; }
}
struct _cffi_align_typedef_ge_cached { char x; ge_cached y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_ge_p1p1(ge_p1p1 *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { int32_t(*tmp)[10] = &p->X; (void)tmp; }
  { int32_t(*tmp)[10] = &p->Y; (void)tmp; }
  { int32_t(*tmp)[10] = &p->Z; (void)tmp; }
  { int32_t(*tmp)[10] = &p->T; (void)tmp; }
}
struct _cffi_align_typedef_ge_p1p1 { char x; ge_p1p1 y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_ge_p2(ge_p2 *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { int32_t(*tmp)[10] = &p->X; (void)tmp; }
  { int32_t(*tmp)[10] = &p->Y; (void)tmp; }
  { int32_t(*tmp)[10] = &p->Z; (void)tmp; }
}
struct _cffi_align_typedef_ge_p2 { char x; ge_p2 y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_ge_p3(ge_p3 *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { int32_t(*tmp)[10] = &p->X; (void)tmp; }
  { int32_t(*tmp)[10] = &p->Y; (void)tmp; }
  { int32_t(*tmp)[10] = &p->Z; (void)tmp; }
  { int32_t(*tmp)[10] = &p->T; (void)tmp; }
}
struct _cffi_align_typedef_ge_p3 { char x; ge_p3 y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_ge_precomp(ge_precomp *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { int32_t(*tmp)[10] = &p->yplusx; (void)tmp; }
  { int32_t(*tmp)[10] = &p->yminusx; (void)tmp; }
  { int32_t(*tmp)[10] = &p->xy2d; (void)tmp; }
}
struct _cffi_align_typedef_ge_precomp { char x; ge_precomp y; };

static int _cffi_d_crypto_sign_ed25519_ref10(unsigned char * x0, unsigned long long * x1, unsigned char const * x2, unsigned long long x3, unsigned char const * x4)
{
  return crypto_sign_ed25519_ref10(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10(PyObject *self, PyObject *args)
{
  unsigned char * x0;
  unsigned long long * x1;
  unsigned char const * x2;
  unsigned long long x3;
  unsigned char const * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(13), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned long long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(13), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, unsigned long long);
  if (x3 == (unsigned long long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = crypto_sign_ed25519_ref10(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10 _cffi_d_crypto_sign_ed25519_ref10
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_fe_0(int32_t * x0)
{
  crypto_sign_ed25519_ref10_fe_0(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_0(PyObject *self, PyObject *arg0)
{
  int32_t * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(72), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(72), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_fe_0(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_0 _cffi_d_crypto_sign_ed25519_ref10_fe_0
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_fe_1(int32_t * x0)
{
  crypto_sign_ed25519_ref10_fe_1(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_1(PyObject *self, PyObject *arg0)
{
  int32_t * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(72), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(72), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_fe_1(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_1 _cffi_d_crypto_sign_ed25519_ref10_fe_1
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_fe_add(int32_t * x0, int32_t const * x1, int32_t const * x2)
{
  crypto_sign_ed25519_ref10_fe_add(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_add(PyObject *self, PyObject *args)
{
  int32_t * x0;
  int32_t const * x1;
  int32_t const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_fe_add", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(72), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(72), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_fe_add(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_add _cffi_d_crypto_sign_ed25519_ref10_fe_add
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_fe_cmov(int32_t * x0, int32_t const * x1, unsigned int x2)
{
  crypto_sign_ed25519_ref10_fe_cmov(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_cmov(PyObject *self, PyObject *args)
{
  int32_t * x0;
  int32_t const * x1;
  unsigned int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_fe_cmov", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(72), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(72), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, unsigned int);
  if (x2 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_fe_cmov(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_cmov _cffi_d_crypto_sign_ed25519_ref10_fe_cmov
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_fe_copy(int32_t * x0, int32_t const * x1)
{
  crypto_sign_ed25519_ref10_fe_copy(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_copy(PyObject *self, PyObject *args)
{
  int32_t * x0;
  int32_t const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_fe_copy", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(72), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(72), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_fe_copy(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_copy _cffi_d_crypto_sign_ed25519_ref10_fe_copy
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_fe_frombytes(int32_t * x0, unsigned char const * x1)
{
  crypto_sign_ed25519_ref10_fe_frombytes(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_frombytes(PyObject *self, PyObject *args)
{
  int32_t * x0;
  unsigned char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_fe_frombytes", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(72), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(72), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_fe_frombytes(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_frombytes _cffi_d_crypto_sign_ed25519_ref10_fe_frombytes
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_fe_invert(int32_t * x0, int32_t const * x1)
{
  crypto_sign_ed25519_ref10_fe_invert(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_invert(PyObject *self, PyObject *args)
{
  int32_t * x0;
  int32_t const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_fe_invert", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(72), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(72), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_fe_invert(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_invert _cffi_d_crypto_sign_ed25519_ref10_fe_invert
#endif

static int _cffi_d_crypto_sign_ed25519_ref10_fe_isnegative(int32_t const * x0)
{
  return crypto_sign_ed25519_ref10_fe_isnegative(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_isnegative(PyObject *self, PyObject *arg0)
{
  int32_t const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = crypto_sign_ed25519_ref10_fe_isnegative(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_isnegative _cffi_d_crypto_sign_ed25519_ref10_fe_isnegative
#endif

static int _cffi_d_crypto_sign_ed25519_ref10_fe_isnonzero(int32_t const * x0)
{
  return crypto_sign_ed25519_ref10_fe_isnonzero(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_isnonzero(PyObject *self, PyObject *arg0)
{
  int32_t const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = crypto_sign_ed25519_ref10_fe_isnonzero(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_isnonzero _cffi_d_crypto_sign_ed25519_ref10_fe_isnonzero
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_fe_mul(int32_t * x0, int32_t const * x1, int32_t const * x2)
{
  crypto_sign_ed25519_ref10_fe_mul(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_mul(PyObject *self, PyObject *args)
{
  int32_t * x0;
  int32_t const * x1;
  int32_t const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_fe_mul", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(72), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(72), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_fe_mul(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_mul _cffi_d_crypto_sign_ed25519_ref10_fe_mul
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_fe_neg(int32_t * x0, int32_t const * x1)
{
  crypto_sign_ed25519_ref10_fe_neg(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_neg(PyObject *self, PyObject *args)
{
  int32_t * x0;
  int32_t const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_fe_neg", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(72), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(72), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_fe_neg(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_neg _cffi_d_crypto_sign_ed25519_ref10_fe_neg
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_fe_pow22523(int32_t * x0, int32_t const * x1)
{
  crypto_sign_ed25519_ref10_fe_pow22523(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_pow22523(PyObject *self, PyObject *args)
{
  int32_t * x0;
  int32_t const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_fe_pow22523", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(72), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(72), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_fe_pow22523(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_pow22523 _cffi_d_crypto_sign_ed25519_ref10_fe_pow22523
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_fe_sq(int32_t * x0, int32_t const * x1)
{
  crypto_sign_ed25519_ref10_fe_sq(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_sq(PyObject *self, PyObject *args)
{
  int32_t * x0;
  int32_t const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_fe_sq", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(72), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(72), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_fe_sq(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_sq _cffi_d_crypto_sign_ed25519_ref10_fe_sq
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_fe_sq2(int32_t * x0, int32_t const * x1)
{
  crypto_sign_ed25519_ref10_fe_sq2(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_sq2(PyObject *self, PyObject *args)
{
  int32_t * x0;
  int32_t const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_fe_sq2", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(72), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(72), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_fe_sq2(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_sq2 _cffi_d_crypto_sign_ed25519_ref10_fe_sq2
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_fe_sub(int32_t * x0, int32_t const * x1, int32_t const * x2)
{
  crypto_sign_ed25519_ref10_fe_sub(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_sub(PyObject *self, PyObject *args)
{
  int32_t * x0;
  int32_t const * x1;
  int32_t const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_fe_sub", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(72), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(72), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_fe_sub(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_sub _cffi_d_crypto_sign_ed25519_ref10_fe_sub
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_fe_tobytes(unsigned char * x0, int32_t const * x1)
{
  crypto_sign_ed25519_ref10_fe_tobytes(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_fe_tobytes(PyObject *self, PyObject *args)
{
  unsigned char * x0;
  int32_t const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_fe_tobytes", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int32_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_fe_tobytes(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_fe_tobytes _cffi_d_crypto_sign_ed25519_ref10_fe_tobytes
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_add(ge_p1p1 * x0, ge_p3 const * x1, ge_cached const * x2)
{
  crypto_sign_ed25519_ref10_ge_add(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_add(PyObject *self, PyObject *args)
{
  ge_p1p1 * x0;
  ge_p3 const * x1;
  ge_cached const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_ge_add", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(23), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_p1p1 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(23), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ge_p3 const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(33), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (ge_cached const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(33), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_add(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_add _cffi_d_crypto_sign_ed25519_ref10_ge_add
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_double_scalarmult_vartime(ge_p2 * x0, unsigned char const * x1, ge_p3 const * x2, unsigned char const * x3)
{
  crypto_sign_ed25519_ref10_ge_double_scalarmult_vartime(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_double_scalarmult_vartime(PyObject *self, PyObject *args)
{
  ge_p2 * x0;
  unsigned char const * x1;
  ge_p3 const * x2;
  unsigned char const * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_ge_double_scalarmult_vartime", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(41), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_p2 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(41), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (ge_p3 const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_double_scalarmult_vartime(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_double_scalarmult_vartime _cffi_d_crypto_sign_ed25519_ref10_ge_double_scalarmult_vartime
#endif

static int _cffi_d_crypto_sign_ed25519_ref10_ge_frombytes_negate_vartime(ge_p3 * x0, unsigned char const * x1)
{
  return crypto_sign_ed25519_ref10_ge_frombytes_negate_vartime(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_frombytes_negate_vartime(PyObject *self, PyObject *args)
{
  ge_p3 * x0;
  unsigned char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_ge_frombytes_negate_vartime", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_p3 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = crypto_sign_ed25519_ref10_ge_frombytes_negate_vartime(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_frombytes_negate_vartime _cffi_d_crypto_sign_ed25519_ref10_ge_frombytes_negate_vartime
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_madd(ge_p1p1 * x0, ge_p3 const * x1, ge_precomp const * x2)
{
  crypto_sign_ed25519_ref10_ge_madd(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_madd(PyObject *self, PyObject *args)
{
  ge_p1p1 * x0;
  ge_p3 const * x1;
  ge_precomp const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_ge_madd", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(23), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_p1p1 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(23), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ge_p3 const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(38), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (ge_precomp const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(38), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_madd(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_madd _cffi_d_crypto_sign_ed25519_ref10_ge_madd
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_msub(ge_p1p1 * x0, ge_p3 const * x1, ge_precomp const * x2)
{
  crypto_sign_ed25519_ref10_ge_msub(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_msub(PyObject *self, PyObject *args)
{
  ge_p1p1 * x0;
  ge_p3 const * x1;
  ge_precomp const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_ge_msub", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(23), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_p1p1 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(23), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ge_p3 const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(38), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (ge_precomp const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(38), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_msub(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_msub _cffi_d_crypto_sign_ed25519_ref10_ge_msub
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_p1p1_to_p2(ge_p2 * x0, ge_p1p1 const * x1)
{
  crypto_sign_ed25519_ref10_ge_p1p1_to_p2(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_p1p1_to_p2(PyObject *self, PyObject *args)
{
  ge_p2 * x0;
  ge_p1p1 const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_ge_p1p1_to_p2", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(41), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_p2 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(41), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(45), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ge_p1p1 const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(45), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_p1p1_to_p2(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_p1p1_to_p2 _cffi_d_crypto_sign_ed25519_ref10_ge_p1p1_to_p2
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_p1p1_to_p3(ge_p3 * x0, ge_p1p1 const * x1)
{
  crypto_sign_ed25519_ref10_ge_p1p1_to_p3(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_p1p1_to_p3(PyObject *self, PyObject *args)
{
  ge_p3 * x0;
  ge_p1p1 const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_ge_p1p1_to_p3", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_p3 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(45), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ge_p1p1 const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(45), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_p1p1_to_p3(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_p1p1_to_p3 _cffi_d_crypto_sign_ed25519_ref10_ge_p1p1_to_p3
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_p2_0(ge_p2 * x0)
{
  crypto_sign_ed25519_ref10_ge_p2_0(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_p2_0(PyObject *self, PyObject *arg0)
{
  ge_p2 * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(41), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_p2 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(41), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_p2_0(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_p2_0 _cffi_d_crypto_sign_ed25519_ref10_ge_p2_0
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_p2_dbl(ge_p1p1 * x0, ge_p2 const * x1)
{
  crypto_sign_ed25519_ref10_ge_p2_dbl(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_p2_dbl(PyObject *self, PyObject *args)
{
  ge_p1p1 * x0;
  ge_p2 const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_ge_p2_dbl", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(23), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_p1p1 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(23), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(24), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ge_p2 const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(24), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_p2_dbl(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_p2_dbl _cffi_d_crypto_sign_ed25519_ref10_ge_p2_dbl
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_p3_0(ge_p3 * x0)
{
  crypto_sign_ed25519_ref10_ge_p3_0(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_p3_0(PyObject *self, PyObject *arg0)
{
  ge_p3 * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_p3 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_p3_0(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_p3_0 _cffi_d_crypto_sign_ed25519_ref10_ge_p3_0
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_p3_dbl(ge_p1p1 * x0, ge_p3 const * x1)
{
  crypto_sign_ed25519_ref10_ge_p3_dbl(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_p3_dbl(PyObject *self, PyObject *args)
{
  ge_p1p1 * x0;
  ge_p3 const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_ge_p3_dbl", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(23), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_p1p1 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(23), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ge_p3 const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_p3_dbl(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_p3_dbl _cffi_d_crypto_sign_ed25519_ref10_ge_p3_dbl
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_p3_to_cached(ge_cached * x0, ge_p3 const * x1)
{
  crypto_sign_ed25519_ref10_ge_p3_to_cached(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_p3_to_cached(PyObject *self, PyObject *args)
{
  ge_cached * x0;
  ge_p3 const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_ge_p3_to_cached", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_cached *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ge_p3 const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_p3_to_cached(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_p3_to_cached _cffi_d_crypto_sign_ed25519_ref10_ge_p3_to_cached
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_p3_to_p2(ge_p2 * x0, ge_p3 const * x1)
{
  crypto_sign_ed25519_ref10_ge_p3_to_p2(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_p3_to_p2(PyObject *self, PyObject *args)
{
  ge_p2 * x0;
  ge_p3 const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_ge_p3_to_p2", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(41), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_p2 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(41), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ge_p3 const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_p3_to_p2(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_p3_to_p2 _cffi_d_crypto_sign_ed25519_ref10_ge_p3_to_p2
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_p3_tobytes(unsigned char * x0, ge_p3 const * x1)
{
  crypto_sign_ed25519_ref10_ge_p3_tobytes(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_p3_tobytes(PyObject *self, PyObject *args)
{
  unsigned char * x0;
  ge_p3 const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_ge_p3_tobytes", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ge_p3 const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_p3_tobytes(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_p3_tobytes _cffi_d_crypto_sign_ed25519_ref10_ge_p3_tobytes
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_precomp_0(ge_precomp * x0)
{
  crypto_sign_ed25519_ref10_ge_precomp_0(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_precomp_0(PyObject *self, PyObject *arg0)
{
  ge_precomp * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(69), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_precomp *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(69), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_precomp_0(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_precomp_0 _cffi_d_crypto_sign_ed25519_ref10_ge_precomp_0
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_scalarmult_base(ge_p3 * x0, unsigned char const * x1)
{
  crypto_sign_ed25519_ref10_ge_scalarmult_base(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_scalarmult_base(PyObject *self, PyObject *args)
{
  ge_p3 * x0;
  unsigned char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_ge_scalarmult_base", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_p3 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_scalarmult_base(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_scalarmult_base _cffi_d_crypto_sign_ed25519_ref10_ge_scalarmult_base
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_sub(ge_p1p1 * x0, ge_p3 const * x1, ge_cached const * x2)
{
  crypto_sign_ed25519_ref10_ge_sub(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_sub(PyObject *self, PyObject *args)
{
  ge_p1p1 * x0;
  ge_p3 const * x1;
  ge_cached const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_ge_sub", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(23), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (ge_p1p1 *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(23), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(20), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ge_p3 const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(20), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(33), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (ge_cached const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(33), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_sub(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_sub _cffi_d_crypto_sign_ed25519_ref10_ge_sub
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_ge_tobytes(unsigned char * x0, ge_p2 const * x1)
{
  crypto_sign_ed25519_ref10_ge_tobytes(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_ge_tobytes(PyObject *self, PyObject *args)
{
  unsigned char * x0;
  ge_p2 const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_ge_tobytes", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(24), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (ge_p2 const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(24), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_ge_tobytes(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_ge_tobytes _cffi_d_crypto_sign_ed25519_ref10_ge_tobytes
#endif

static int _cffi_d_crypto_sign_ed25519_ref10_keypair(unsigned char * x0, unsigned char * x1)
{
  return crypto_sign_ed25519_ref10_keypair(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_keypair(PyObject *self, PyObject *args)
{
  unsigned char * x0;
  unsigned char * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_keypair", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = crypto_sign_ed25519_ref10_keypair(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_keypair _cffi_d_crypto_sign_ed25519_ref10_keypair
#endif

static int _cffi_d_crypto_sign_ed25519_ref10_open(unsigned char * x0, unsigned long long * x1, unsigned char const * x2, unsigned long long x3, unsigned char const * x4)
{
  return crypto_sign_ed25519_ref10_open(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_open(PyObject *self, PyObject *args)
{
  unsigned char * x0;
  unsigned long long * x1;
  unsigned char const * x2;
  unsigned long long x3;
  unsigned char const * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_open", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(13), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned long long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(13), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, unsigned long long);
  if (x3 == (unsigned long long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = crypto_sign_ed25519_ref10_open(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_open _cffi_d_crypto_sign_ed25519_ref10_open
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_sc_muladd(unsigned char * x0, unsigned char const * x1, unsigned char const * x2, unsigned char const * x3)
{
  crypto_sign_ed25519_ref10_sc_muladd(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_sc_muladd(PyObject *self, PyObject *args)
{
  unsigned char * x0;
  unsigned char const * x1;
  unsigned char const * x2;
  unsigned char const * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "crypto_sign_ed25519_ref10_sc_muladd", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_sc_muladd(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_sc_muladd _cffi_d_crypto_sign_ed25519_ref10_sc_muladd
#endif

static void _cffi_d_crypto_sign_ed25519_ref10_sc_reduce(unsigned char * x0)
{
  crypto_sign_ed25519_ref10_sc_reduce(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_crypto_sign_ed25519_ref10_sc_reduce(PyObject *self, PyObject *arg0)
{
  unsigned char * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { crypto_sign_ed25519_ref10_sc_reduce(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_crypto_sign_ed25519_ref10_sc_reduce _cffi_d_crypto_sign_ed25519_ref10_sc_reduce
#endif

static int _cffi_const_CRYPTO_BYTES(unsigned long long *o)
{
  int n = (CRYPTO_BYTES) <= 0;
  *o = (unsigned long long)((CRYPTO_BYTES) | 0);  /* check that CRYPTO_BYTES is an integer */
  if (!_cffi_check_int(*o, n, 64U))
    n |= 2;
  return n;
}

static int _cffi_const_CRYPTO_PUBLICKEYBYTES(unsigned long long *o)
{
  int n = (CRYPTO_PUBLICKEYBYTES) <= 0;
  *o = (unsigned long long)((CRYPTO_PUBLICKEYBYTES) | 0);  /* check that CRYPTO_PUBLICKEYBYTES is an integer */
  if (!_cffi_check_int(*o, n, 32U))
    n |= 2;
  return n;
}

static int _cffi_const_CRYPTO_SECRETKEYBYTES(unsigned long long *o)
{
  int n = (CRYPTO_SECRETKEYBYTES) <= 0;
  *o = (unsigned long long)((CRYPTO_SECRETKEYBYTES) | 0);  /* check that CRYPTO_SECRETKEYBYTES is an integer */
  if (!_cffi_check_int(*o, n, 64U))
    n |= 2;
  return n;
}

static int _cffi_const_crypto_sign_ed25519_ref10_BYTES(unsigned long long *o)
{
  int n = (crypto_sign_ed25519_ref10_BYTES) <= 0;
  *o = (unsigned long long)((crypto_sign_ed25519_ref10_BYTES) | 0);  /* check that crypto_sign_ed25519_ref10_BYTES is an integer */
  if (!_cffi_check_int(*o, n, 64U))
    n |= 2;
  return n;
}

static int _cffi_const_crypto_sign_ed25519_ref10_PUBLICKEYBYTES(unsigned long long *o)
{
  int n = (crypto_sign_ed25519_ref10_PUBLICKEYBYTES) <= 0;
  *o = (unsigned long long)((crypto_sign_ed25519_ref10_PUBLICKEYBYTES) | 0);  /* check that crypto_sign_ed25519_ref10_PUBLICKEYBYTES is an integer */
  if (!_cffi_check_int(*o, n, 32U))
    n |= 2;
  return n;
}

static int _cffi_const_crypto_sign_ed25519_ref10_SECRETKEYBYTES(unsigned long long *o)
{
  int n = (crypto_sign_ed25519_ref10_SECRETKEYBYTES) <= 0;
  *o = (unsigned long long)((crypto_sign_ed25519_ref10_SECRETKEYBYTES) | 0);  /* check that crypto_sign_ed25519_ref10_SECRETKEYBYTES is an integer */
  if (!_cffi_check_int(*o, n, 64U))
    n |= 2;
  return n;
}

static const struct _cffi_global_s _cffi_globals[] = {
  { "CRYPTO_BYTES", (void *)_cffi_const_CRYPTO_BYTES, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "CRYPTO_PUBLICKEYBYTES", (void *)_cffi_const_CRYPTO_PUBLICKEYBYTES, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "CRYPTO_SECRETKEYBYTES", (void *)_cffi_const_CRYPTO_SECRETKEYBYTES, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "crypto_sign_ed25519_ref10", (void *)_cffi_f_crypto_sign_ed25519_ref10, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 11), (void *)_cffi_d_crypto_sign_ed25519_ref10 },
  { "crypto_sign_ed25519_ref10_BYTES", (void *)_cffi_const_crypto_sign_ed25519_ref10_BYTES, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "crypto_sign_ed25519_ref10_PUBLICKEYBYTES", (void *)_cffi_const_crypto_sign_ed25519_ref10_PUBLICKEYBYTES, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "crypto_sign_ed25519_ref10_SECRETKEYBYTES", (void *)_cffi_const_crypto_sign_ed25519_ref10_SECRETKEYBYTES, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "crypto_sign_ed25519_ref10_fe_0", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_0, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 71), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_0 },
  { "crypto_sign_ed25519_ref10_fe_1", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_1, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 71), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_1 },
  { "crypto_sign_ed25519_ref10_fe_add", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_add, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 78), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_add },
  { "crypto_sign_ed25519_ref10_fe_cmov", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_cmov, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 83), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_cmov },
  { "crypto_sign_ed25519_ref10_fe_copy", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_copy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 74), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_copy },
  { "crypto_sign_ed25519_ref10_fe_frombytes", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_frombytes, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 88), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_frombytes },
  { "crypto_sign_ed25519_ref10_fe_invert", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_invert, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 74), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_invert },
  { "crypto_sign_ed25519_ref10_fe_isnegative", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_isnegative, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 4), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_isnegative },
  { "crypto_sign_ed25519_ref10_fe_isnonzero", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_isnonzero, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 4), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_isnonzero },
  { "crypto_sign_ed25519_ref10_fe_mul", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_mul, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 78), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_mul },
  { "crypto_sign_ed25519_ref10_fe_neg", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_neg, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 74), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_neg },
  { "crypto_sign_ed25519_ref10_fe_pow22523", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_pow22523, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 74), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_pow22523 },
  { "crypto_sign_ed25519_ref10_fe_sq", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_sq, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 74), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_sq },
  { "crypto_sign_ed25519_ref10_fe_sq2", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_sq2, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 74), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_sq2 },
  { "crypto_sign_ed25519_ref10_fe_sub", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_sub, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 78), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_sub },
  { "crypto_sign_ed25519_ref10_fe_tobytes", (void *)_cffi_f_crypto_sign_ed25519_ref10_fe_tobytes, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 103), (void *)_cffi_d_crypto_sign_ed25519_ref10_fe_tobytes },
  { "crypto_sign_ed25519_ref10_ge_add", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_add, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 30), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_add },
  { "crypto_sign_ed25519_ref10_ge_double_scalarmult_vartime", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_double_scalarmult_vartime, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 51), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_double_scalarmult_vartime },
  { "crypto_sign_ed25519_ref10_ge_frombytes_negate_vartime", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_frombytes_negate_vartime, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_frombytes_negate_vartime },
  { "crypto_sign_ed25519_ref10_ge_madd", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_madd, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 35), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_madd },
  { "crypto_sign_ed25519_ref10_ge_msub", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_msub, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 35), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_msub },
  { "crypto_sign_ed25519_ref10_ge_p1p1_to_p2", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_p1p1_to_p2, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 43), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_p1p1_to_p2 },
  { "crypto_sign_ed25519_ref10_ge_p1p1_to_p3", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_p1p1_to_p3, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 60), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_p1p1_to_p3 },
  { "crypto_sign_ed25519_ref10_ge_p2_0", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_p2_0, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 40), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_p2_0 },
  { "crypto_sign_ed25519_ref10_ge_p2_dbl", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_p2_dbl, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 22), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_p2_dbl },
  { "crypto_sign_ed25519_ref10_ge_p3_0", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_p3_0, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 57), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_p3_0 },
  { "crypto_sign_ed25519_ref10_ge_p3_dbl", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_p3_dbl, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 26), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_p3_dbl },
  { "crypto_sign_ed25519_ref10_ge_p3_to_cached", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_p3_to_cached, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 18), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_p3_to_cached },
  { "crypto_sign_ed25519_ref10_ge_p3_to_p2", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_p3_to_p2, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 47), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_p3_to_p2 },
  { "crypto_sign_ed25519_ref10_ge_p3_tobytes", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_p3_tobytes, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 99), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_p3_tobytes },
  { "crypto_sign_ed25519_ref10_ge_precomp_0", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_precomp_0, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 68), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_precomp_0 },
  { "crypto_sign_ed25519_ref10_ge_scalarmult_base", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_scalarmult_base, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 64), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_scalarmult_base },
  { "crypto_sign_ed25519_ref10_ge_sub", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_sub, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 30), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_sub },
  { "crypto_sign_ed25519_ref10_ge_tobytes", (void *)_cffi_f_crypto_sign_ed25519_ref10_ge_tobytes, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 95), (void *)_cffi_d_crypto_sign_ed25519_ref10_ge_tobytes },
  { "crypto_sign_ed25519_ref10_keypair", (void *)_cffi_f_crypto_sign_ed25519_ref10_keypair, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 7), (void *)_cffi_d_crypto_sign_ed25519_ref10_keypair },
  { "crypto_sign_ed25519_ref10_open", (void *)_cffi_f_crypto_sign_ed25519_ref10_open, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 11), (void *)_cffi_d_crypto_sign_ed25519_ref10_open },
  { "crypto_sign_ed25519_ref10_sc_muladd", (void *)_cffi_f_crypto_sign_ed25519_ref10_sc_muladd, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 107), (void *)_cffi_d_crypto_sign_ed25519_ref10_sc_muladd },
  { "crypto_sign_ed25519_ref10_sc_reduce", (void *)_cffi_f_crypto_sign_ed25519_ref10_sc_reduce, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 92), (void *)_cffi_d_crypto_sign_ed25519_ref10_sc_reduce },
};

static const struct _cffi_field_s _cffi_fields[] = {
  { "YplusX", offsetof(ge_cached, YplusX),
              sizeof(((ge_cached *)0)->YplusX),
              _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "YminusX", offsetof(ge_cached, YminusX),
               sizeof(((ge_cached *)0)->YminusX),
               _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "Z", offsetof(ge_cached, Z),
         sizeof(((ge_cached *)0)->Z),
         _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "T2d", offsetof(ge_cached, T2d),
           sizeof(((ge_cached *)0)->T2d),
           _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "X", offsetof(ge_p1p1, X),
         sizeof(((ge_p1p1 *)0)->X),
         _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "Y", offsetof(ge_p1p1, Y),
         sizeof(((ge_p1p1 *)0)->Y),
         _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "Z", offsetof(ge_p1p1, Z),
         sizeof(((ge_p1p1 *)0)->Z),
         _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "T", offsetof(ge_p1p1, T),
         sizeof(((ge_p1p1 *)0)->T),
         _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "X", offsetof(ge_p2, X),
         sizeof(((ge_p2 *)0)->X),
         _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "Y", offsetof(ge_p2, Y),
         sizeof(((ge_p2 *)0)->Y),
         _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "Z", offsetof(ge_p2, Z),
         sizeof(((ge_p2 *)0)->Z),
         _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "X", offsetof(ge_p3, X),
         sizeof(((ge_p3 *)0)->X),
         _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "Y", offsetof(ge_p3, Y),
         sizeof(((ge_p3 *)0)->Y),
         _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "Z", offsetof(ge_p3, Z),
         sizeof(((ge_p3 *)0)->Z),
         _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "T", offsetof(ge_p3, T),
         sizeof(((ge_p3 *)0)->T),
         _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "yplusx", offsetof(ge_precomp, yplusx),
              sizeof(((ge_precomp *)0)->yplusx),
              _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "yminusx", offsetof(ge_precomp, yminusx),
               sizeof(((ge_precomp *)0)->yminusx),
               _CFFI_OP(_CFFI_OP_NOOP, 119) },
  { "xy2d", offsetof(ge_precomp, xy2d),
            sizeof(((ge_precomp *)0)->xy2d),
            _CFFI_OP(_CFFI_OP_NOOP, 119) },
};

static const struct _cffi_struct_union_s _cffi_struct_unions[] = {
  { "$ge_cached", 113, _CFFI_F_CHECK_FIELDS,
    sizeof(ge_cached), offsetof(struct _cffi_align_typedef_ge_cached, y), 0, 4 },
  { "$ge_p1p1", 114, _CFFI_F_CHECK_FIELDS,
    sizeof(ge_p1p1), offsetof(struct _cffi_align_typedef_ge_p1p1, y), 4, 4 },
  { "$ge_p2", 115, _CFFI_F_CHECK_FIELDS,
    sizeof(ge_p2), offsetof(struct _cffi_align_typedef_ge_p2, y), 8, 3 },
  { "$ge_p3", 116, _CFFI_F_CHECK_FIELDS,
    sizeof(ge_p3), offsetof(struct _cffi_align_typedef_ge_p3, y), 11, 4 },
  { "$ge_precomp", 117, _CFFI_F_CHECK_FIELDS,
    sizeof(ge_precomp), offsetof(struct _cffi_align_typedef_ge_precomp, y), 15, 3 },
};

static const struct _cffi_typename_s _cffi_typenames[] = {
  { "crypto_int32", 118 },
  { "fe", 119 },
  { "ge_cached", 113 },
  { "ge_p1p1", 114 },
  { "ge_p2", 115 },
  { "ge_p3", 116 },
  { "ge_precomp", 117 },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  _cffi_fields,
  _cffi_struct_unions,
  NULL,  /* no enums */
  _cffi_typenames,
  45,  /* num_globals */
  5,  /* num_struct_unions */
  0,  /* num_enums */
  7,  /* num_typenames */
  NULL,  /* no includes */
  124,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit__crypto_sign(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit__crypto_sign(void) { return NULL; }
#  else
     init_crypto_sign(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit__crypto_sign(void)
{
  return _cffi_init("_crypto_sign", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
init_crypto_sign(void)
{
  _cffi_init("_crypto_sign", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
